@@ -1,4 +1,4 @@
-/*	$OpenBSD: extern.h,v 1.226 2024/08/21 19:35:31 job Exp $ */
+/*	$OpenBSD: extern.h,v 1.228 2024/09/12 10:33:25 tb Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -916,7 +916,10 @@ char		*x509_get_pubkey(X509 *, const char *);
 char		*x509_pubkey_get_ski(X509_PUBKEY *, const char *);
 enum cert_purpose	 x509_get_purpose(X509 *, const char *);
 int		 x509_get_time(const ASN1_TIME *, time_t *);
-char		*x509_convert_seqnum(const char *, const ASN1_INTEGER *);
+char		*x509_convert_seqnum(const char *, const char *,
+		    const ASN1_INTEGER *);
+int		 x509_valid_seqnum(const char *, const char *,
+		    const ASN1_INTEGER *);
 int		 x509_location(const char *, const char *, GENERAL_NAME *,
 		    char **);
 int		 x509_inherits(X509 *);
@@ -1002,6 +1005,9 @@ int	mkpathat(int, const char *);
 
 #define DEFAULT_SKIPLIST_FILE	RPKI_PATH_TAL_DIR "/skiplist"
 
+/* Interval in which random reinitialization to an RRDP snapshot happens. */
+#define RRDP_RANDOM_REINIT_MAX	12 /* weeks */
+
 /* Maximum number of TAL files we'll load. */
 #define	TALSZ_MAX		8
 #define	CERTID_MAX		1000000
