@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.264 2024/08/20 13:31:49 claudio Exp $ */
+/*	$OpenBSD: main.c,v 1.266 2024/09/04 15:46:43 job Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -1009,7 +1009,7 @@ main(int argc, char *argv[])
 	    "proc exec unveil", NULL) == -1)
 		err(1, "pledge");
 
-	while ((c = getopt(argc, argv, "Ab:Bcd:e:fH:jmnoP:rRs:S:t:T:vVx")) != -1)
+	while ((c = getopt(argc, argv, "Ab:Bcd:e:fH:jmnoP:Rs:S:t:T:vVx")) != -1)
 		switch (c) {
 		case 'A':
 			excludeaspa = 1;
@@ -1058,9 +1058,6 @@ main(int argc, char *argv[])
 		case 'R':
 			rrdpon = 0;
 			break;
-		case 'r': /* Remove after OpenBSD 7.3 */
-			rrdpon = 1;
-			break;
 		case 's':
 			timeout = strtonum(optarg, 0, 24*60*60, &errs);
 			if (errs)
@@ -1491,9 +1488,12 @@ main(int argc, char *argv[])
 	    "invalid)\n", stats.repo_tal_stats.aspas,
 	    stats.repo_tal_stats.aspas_fail,
 	    stats.repo_tal_stats.aspas_invalid);
-	printf("Signed Prefix Lists: %u (%u failed parse, %u invalid)\n",
-	    stats.repo_tal_stats.spls, stats.repo_tal_stats.spls_fail,
-	    stats.repo_tal_stats.spls_invalid);
+	if (experimental) {
+		printf("Signed Prefix Lists: %u "
+		    "(%u failed parse, %u invalid)\n",
+		    stats.repo_tal_stats.spls, stats.repo_tal_stats.spls_fail,
+		    stats.repo_tal_stats.spls_invalid);
+	}
 	printf("BGPsec Router Certificates: %u\n", stats.repo_tal_stats.brks);
 	printf("Certificates: %u (%u invalid)\n",
 	    stats.repo_tal_stats.certs, stats.repo_tal_stats.certs_fail);
@@ -1526,7 +1526,7 @@ main(int argc, char *argv[])
 
 usage:
 	fprintf(stderr,
-	    "usage: rpki-client [-ABcjmnoRrVvx] [-b sourceaddr] [-d cachedir]"
+	    "usage: rpki-client [-ABcjmnoRVvx] [-b sourceaddr] [-d cachedir]"
 	    " [-e rsync_prog]\n"
 	    "                   [-H fqdn] [-P epoch] [-S skiplist] [-s timeout]"
 	    " [-T table]\n"
